package org.klomp.eclipse.fsize;

import java.io.File;
import java.text.FieldPosition;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;

import org.eclipse.swt.widgets.Display;
import org.eclipse.jface.viewers.IDecoration;
import org.eclipse.jface.viewers.ILightweightLabelDecorator;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.LabelProviderChangedEvent;

/**
 * Decorates files with their size (if known). A file decorator, used in tree
 * view like the Navigator and Java Package views. The act of decoration itself
 * is straightforward, but the logic that is needed to automatically update the
 * decoration when a file changes is more complicated.
 * 
 * @author Erwin Bolwidt
 */
public class FileSizeDecorator
    extends LabelProvider
    implements ILightweightLabelDecorator, IResourceChangeListener {

    public final static String DECORATOR_ID = "decorator";

    public FileSizeDecorator() {
        IWorkspace workspace = ResourcesPlugin.getWorkspace();
        workspace.addResourceChangeListener(
            this,
            IResourceChangeEvent.POST_CHANGE);
    }

    /**
     * Creates the decoration for an element when asked by the workbench ui.
     */
    public void decorate(Object element, IDecoration decoration) {
        if (!(element instanceof IFile))
            return;

        FileSizeDecoratorPlugin plugin = FileSizeDecoratorPlugin.getDefault();

        MessageFormat format = plugin.getFormat();
        if (format == null)
            return;

        IFile file = (IFile) element;

        long size = getSize(file);
        if (size == -1L)
            return;

        int kiloFactor = plugin.getKiloFactor();
        if (kiloFactor == 0)
            kiloFactor = 1024;

        long megaFactor = (long) kiloFactor * (long) kiloFactor;
        long gigaFactor =
            (long) kiloFactor * (long) kiloFactor * (long) kiloFactor;

        String unit;

        // Add half of unit before dividing, to ensure proper rounding 
        // to the nearest unit.
        if (size >= gigaFactor) {
            unit = plugin.getGigaString();
            size = (size + gigaFactor / 2) / gigaFactor;
        } else if (size >= megaFactor) {
            unit = plugin.getMegaString();
            size = (size + megaFactor / 2) / megaFactor;
        } else if (size >= kiloFactor) {
            unit = plugin.getKiloString();
            size = (size + kiloFactor / 2) / kiloFactor;
        } else {
            unit = plugin.getBytesString();
        }

        StringBuffer decorateString =
            format.format(
                new Object[] { new Long(size), unit },
                new StringBuffer(),
                new FieldPosition(0));

        if (plugin.getLocation() == FileSizeDecoratorPlugin.LOCATION_PREFIX)
            decoration.addPrefix(decorateString + " ");
        else
            decoration.addSuffix(" " + decorateString);
    }

    /**
     * Returns the size of the given file when it is easy to calculate. When it
     * is not easy or impossible to calculate, it returns -1L. "Not easy" means
     * that it would take very long, because it would require reading the file
     * to determine the size.
     */
    private long getSize(IFile file) {
        if (!file.isLocal(IResource.DEPTH_ZERO))
            return -1L;

        IPath location = file.getLocation();
        if (location == null) {
            return -1L;
        } else {
            File localFile = location.toFile();
            return localFile.length();
        }
    }

    /**
     * Called by the workspace after a resource change. This method in turn
     * generates UI events that trigger an update of the decorations of resource
     * that this decorator can handle.
     */
    public void resourceChanged(IResourceChangeEvent event) {
        IResourceDelta[] deltas = new IResourceDelta[] { event.getDelta()};
        List changes = new ArrayList();

        collectChangedFiles(changes, deltas);

        if (!changes.isEmpty()) {
            postLabelEvent(
                new LabelProviderChangedEvent(this, changes.toArray()));
        }
    }

    private void collectChangedFiles(
        Collection collector,
        IResourceDelta[] deltas) {

        for (int i = 0; i < deltas.length; i++) {
            if ((deltas[i].getKind() & IResourceDelta.CHANGED) != 0) {
                IResource resource = deltas[i].getResource();
                if (resource instanceof IFile) {
                    if (getSize((IFile) resource) != -1L)
                        collector.add(resource);
                }
            }
            collectChangedFiles(
                collector,
                deltas[i].getAffectedChildren(IResourceDelta.CHANGED));
        }
    }

    /**
     * Post the label event to the UI thread.
     *
     * @param event The event to post.
     */
    private void postLabelEvent(final LabelProviderChangedEvent event) {
        Display.getDefault().asyncExec(new Runnable() {
            public void run() {
                fireLabelProviderChanged(event);
            }
        });
    }

    public static void refresh() {
        FileSizeDecorator decorator =
            FileSizeDecoratorPlugin.getActiveFileSizeDecorator();

        if (decorator == null)
            return;

        // update decoration of all resources
        decorator.postLabelEvent(new LabelProviderChangedEvent(decorator));
    }
}