package org.klomp.eclipse.fsize;

import java.text.MessageFormat;
import java.util.ResourceBundle;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPluginDescriptor;

import org.eclipse.ui.IDecoratorManager;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import org.eclipse.jface.preference.IPreferenceStore;

/**
 * File-size decorator central plugin class. 
 * 
 * @author Erwin Bolwidt
 */
public class FileSizeDecoratorPlugin extends AbstractUIPlugin {
    public final static int LOCATION_PREFIX = 1;
    public final static int LOCATION_SUFFIX = 0;

    private final static String DEFAULT_FORMAT_STRING = "- {0} {1}";

    /**
     * The shared instance.
     */
    private static FileSizeDecoratorPlugin plugin;

    private String formatString;
    private String bytesString;
    private String kiloString;
    private String megaString;
    private String gigaString;
    private int kiloFactor;
    private int location;

    private MessageFormat format;

    public FileSizeDecoratorPlugin(IPluginDescriptor descriptor) {
        super(descriptor);
        if (plugin != null)
            throw new IllegalStateException(
                "Plugin instantiated twice: " + descriptor.getUniqueIdentifier());

        plugin = this;
    }

    /**
     * Returns the shared instance.
     */
    public static FileSizeDecoratorPlugin getDefault() {
        return plugin;
    }

    /**
     * Returns the string from the plugin's resource bundle,
     * or <code>key</code> if not found.
     */
    public static String getResourceString(String key) {
        return getPluginDescriptor().getResourceString(key);
    }

    public static IPluginDescriptor getPluginDescriptor() {
        return getDefault().getDescriptor();
    }

    /**
     * Returns the plugin's resource bundle,
     */
    public ResourceBundle getResourceBundle() {
        return getDescriptor().getResourceBundle();
    }

    public static FileSizeDecorator getActiveFileSizeDecorator() {
        IDecoratorManager manager =
            getDefault().getWorkbench().getDecoratorManager();
        String decoratorId =
            getPluginDescriptor().getUniqueIdentifier()
                + '.'
                + FileSizeDecorator.DECORATOR_ID;

        if (manager.getEnabled(decoratorId))
            return (FileSizeDecorator) manager.getLightweightLabelDecorator(
                decoratorId);
        return null;
    }

    public String getBytesString() {
        return bytesString;
    }

    public String getFormatString() {
        return formatString;
    }

    public MessageFormat getFormat() {
        return format;
    }

    public String getGigaString() {
        return gigaString;
    }

    public int getKiloFactor() {
        return kiloFactor;
    }

    public String getKiloString() {
        return kiloString;
    }

    public String getMegaString() {
        return megaString;
    }

    public void setBytesString(String bytesString) {
        this.bytesString = bytesString;
    }

    public void setFormatString(String formatString) {
        this.formatString = formatString;
        try {
            this.format = new MessageFormat(formatString);
        } catch (IllegalArgumentException e) {
            this.format = new MessageFormat(DEFAULT_FORMAT_STRING);
        }
    }

    public void setGigaString(String gigaString) {
        this.gigaString = gigaString;
    }

    public void setKiloFactor(int kiloFactor) {
        this.kiloFactor = kiloFactor;
    }

    public void setKiloString(String kiloString) {
        this.kiloString = kiloString;
    }

    public void setMegaString(String megaString) {
        this.megaString = megaString;
    }

    public void startup() throws CoreException {
        super.startup();

        IPreferenceStore store = getPreferenceStore();

        store.setDefault(
            FSDPreferencePage.PREF_FORMAT_STRING,
            DEFAULT_FORMAT_STRING);
        store.setDefault(FSDPreferencePage.PREF_LOCATION, LOCATION_SUFFIX);
        store.setDefault(FSDPreferencePage.PREF_BYTES_STRING, "bytes");
        store.setDefault(FSDPreferencePage.PREF_KILO_STRING, "Kb");
        store.setDefault(FSDPreferencePage.PREF_MEGA_STRING, "Mb");
        store.setDefault(FSDPreferencePage.PREF_GIGA_STRING, "Gb");
        store.setDefault(FSDPreferencePage.PREF_KILO_FACTOR, 1024);

        setFormatString(store.getString(FSDPreferencePage.PREF_FORMAT_STRING));
        setLocation(store.getInt(FSDPreferencePage.PREF_LOCATION));
        setBytesString(store.getString(FSDPreferencePage.PREF_BYTES_STRING));
        setKiloString(store.getString(FSDPreferencePage.PREF_KILO_STRING));
        setMegaString(store.getString(FSDPreferencePage.PREF_MEGA_STRING));
        setGigaString(store.getString(FSDPreferencePage.PREF_GIGA_STRING));
        setKiloFactor(store.getInt(FSDPreferencePage.PREF_KILO_FACTOR));
    }

    public int getLocation() {
        return location;
    }

    public void setLocation(int location) {
        this.location = location;
    }

}
